
package com.redxun.knowledge.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.entity.IUser;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.utils.ContextUtil;
import com.redxun.dto.user.OsUserDto;
import com.redxun.feign.sys.SystemClient;
import com.redxun.knowledge.analysis.entity.dao.PvLog;
import com.redxun.knowledge.analysis.entity.dto.SystemLogDto;
import com.redxun.knowledge.analysis.entity.vo.KnowledgeTypeParse;
import com.redxun.knowledge.analysis.mapper.PvLogMapper;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.CommonEntity;
import com.redxun.knowledge.entity.dao.OsGroup;
import com.redxun.knowledge.entity.vo.DicVo;
import com.redxun.knowledge.entity.vo.PositionSequenceVo;
import com.redxun.knowledge.mapper.CommonMapper;
import com.redxun.search.service.impl.SearchServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 业务系统相关服务类
 */
@Slf4j
@Service
public class CommonServiceImpl extends SuperServiceImpl<CommonMapper, CommonEntity> implements BaseService<CommonEntity> {

    @Resource
    private CommonMapper commonMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private PvLogMapper pvLogMapper;

    @Autowired
    private SearchServiceImpl searchService;

    @Resource
    @Lazy
    SystemClient systemClient;

    @Override
    public BaseDao<CommonEntity> getRepository() {
        return commonMapper;
    }

    /**
     * 查询值级列表
     *
     * @return
     */
    public List<String> queryPositionGrade() {
//        DataSourceContextHolder.setDataSource("train");
        List<String> result = commonMapper.queryPositionGrade();

        // 调用完成后清除当前数据源
//        DataSourceContextHolder.clearDataSource();

        return result;
    }

    public List<PositionSequenceVo> queryPositionSequence() {
//        DataSourceContextHolder.setDataSource("train");
        List<PositionSequenceVo> result = commonMapper.queryPositionSequence();
        // 调用完成后清除当前数据源
//        DataSourceContextHolder.clearDataSource();
        return result;
    }

    /**
     * 获取字典数据(知识地图类型)
     *
     * @param alias
     * @return
     */
    public List<DicVo> queryDic(String alias) {
        List<DicVo> result = commonMapper.queryDic(alias);
        return result;
    }

    /**
     * 获取当前登录用户信息
     *
     * @return
     */
    public Map<String, Object> loginUserInfo() {
        String userId = userService.queryLoginUser();
        OsUserDto osUserDto = userService.queryOsUserDto(userId);
        Map<String, Object> map = new HashMap<>();
        map.put("userId", userId);
        map.put("name", osUserDto.getFullName());
        map.put("companyName", null);
        map.put("deptName", osUserDto.getDeptName());
        return map;
    }

    /**
     * 用户浏览估计上报接口
     *
     * @param systemLogDto
     */
    public void pageTrack(SystemLogDto systemLogDto) {
        IUser users = ContextUtil.getCurrentUser();
        PvLog pvLog = new PvLog();
        //专辑详情&地图详情补充字段处理
        if (("知识专辑").equals(systemLogDto.getModule()) && ("详情").equals(systemLogDto.getSubModule()) ||
                ("知识地图").equals(systemLogDto.getModule()) && ("详情").equals(systemLogDto.getSubModule())) {
            String ussd = systemLogDto.getUssd();
            KnowledgeTypeParse knowledgeTypeParse = JSON.parseObject(ussd, KnowledgeTypeParse.class);
            String id = knowledgeTypeParse.getId();
            systemLogDto.setUssd(id);
        }
        BeanUtils.copyProperties(systemLogDto, pvLog);
        pvLog.setPkId(IdGenerator.getIdStr());
        pvLog.setCompanyId(systemLogDto.getCompanyId());
        pvLog.setCreateDepId(users.getDeptId());
        pvLog.setTenantId(users.getTenantId());
        pvLog.setCreateBy(users.getUserId());
        pvLog.setCreateTime(new Date());
        pvLog.setUpdateBy(users.getUserId());
        pvLog.setUpdateTime(new Date());
        pvLogMapper.insert(pvLog);
    }

    /**
     * 搜索关键字上报
     *
     * @param keyword
     */
    public void searchKeywordTrack(String keyword) {
        IUser users = ContextUtil.getCurrentUser();
        searchService.keywordParticiple(keyword, users);
    }


    @Override
    public List<OsGroup> queryList(QueryFilter queryFilter) {
        BaseDao baseDao = this.getRepository();
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        List<OsGroup> query = baseDao.query(params);
        return query;
    }
}
